#include <stdbool.h>
#include "CustomerRegister.h"
#include "Inventory.h"
#include "OrderProcess.h"
#include <string.h>

static const double SMALL_DISCOUNT = 0.75;
static const double MEDIUM_DISCOUNT = 0.50;
static const double MAJOR_DISCOUNT = 0.25;

static const int FREE_SHIPPING_LIMIT = 700;

static const int ORDER_SIZE = 3;

bool checkInventoryAndValidateCustomer(Item items[], Customer customer) {
	bool isItemInStock = false;

	int counter = 0;
	for (int i = 0; i < ORDER_SIZE; i++) {
		for (int j = 0; j < INVENTORY_SIZE; j++) {
			if (inventory[j].itemNo == items[i].itemNo) {
				counter++;
			}
		}
	}
	isItemInStock = counter == ORDER_SIZE;

	bool isCustomerValid = false;
	for (int i = 0; i < CUSTOMERREGISTER_SIZE; i++) {
		if (customerRegister[i].custNo == customer.custNo) {
			if (customerRegister[i].status == GOOD) {
				isCustomerValid = true;
			}
		}
	}

	return isItemInStock && isCustomerValid;
}

float give25PercentDiscount(float total) {
	float percentage = SMALL_DISCOUNT;
	return total * percentage;
}

float give50PercentDiscount(float total) {
	float percentage = MEDIUM_DISCOUNT;
	return total * percentage;
}

float give75PercentDiscount(float total) {
	float percentage = MAJOR_DISCOUNT;
	return total * percentage;
}

float calculatePriceAndApplyDiscounts(Item items[]) {
	float runningTotal = 0.0;

	for (int i = 0; i < ORDER_SIZE; ++i) {
		if (items[i].type == AMETHYST) {
			runningTotal += items[i].carat * 12000;
		} else {
			if (items[i].type == DIAMOND) {
				runningTotal += items[i].carat * 16000;
			} else {
				if (items[i].type == EMERALD) {
					runningTotal += items[i].carat * 13000;
				} else {
					if (items[i].type == RUBY) {
						runningTotal += items[i].carat * 13200;
					} else {
						if (items[i].type == SAPPHIRE) {
							runningTotal += items[i].carat * 10100;
						}
					}
				}
			}
		}
	}

	if (runningTotal > 20000) {
		runningTotal = give75PercentDiscount(runningTotal);
	} else if (runningTotal > 10000) {
		runningTotal = give50PercentDiscount(runningTotal);
	} else if (runningTotal > 5000) {
		runningTotal = give25PercentDiscount(runningTotal);
	}

	return runningTotal;
}

float processOrder(Item items[], Customer customer) {
	//1. step - validate
	checkInventoryAndValidateCustomer(items, customer);

	//2. step - calculate Price and discount
	float total = calculatePriceAndApplyDiscounts(items);

	//3. step - apply shipping charge
	if (total < FREE_SHIPPING_LIMIT) {
		float shippingFee = 70;
		total = total + shippingFee;
	}
	return total;
}
